"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const _1 = require("./");
const c = require("./config");
const utils = require("egret-node-utils");
const fs = require("fs-extra-promise");
const path = require("path");
let projectRoot;
let resourcePath;
function build(p, format) {
    return __awaiter(this, void 0, void 0, function* () {
        let result = yield _1.ResourceConfig.init(p);
        _1.ResourceConfig.typeSelector = result.typeSelector;
        if (!_1.ResourceConfig.typeSelector) {
            throw "missing typeSelector in Main.ts";
        }
        let mergeCollection = {};
        let executeFilter = (f) => __awaiter(this, void 0, void 0, function* () {
            let config = _1.ResourceConfig.config;
            merge.walk(f);
            // var ext = f.substr(f.lastIndexOf(".") + 1);
            let type = _1.ResourceConfig.typeSelector(f);
            if (type) {
                return { name: f, url: f, type };
            }
        });
        projectRoot = p;
        resourcePath = path.join(projectRoot, result.resourceRoot);
        let filename = path.join(resourcePath, result.resourceConfigFileName);
        let option = {
            relative: true,
            ignoreHiddenFile: true
        };
        let list = yield utils.walk(resourcePath, () => true, option);
        let files = yield Promise.all(list.map(executeFilter));
        files.filter(a => a).forEach(element => _1.ResourceConfig.addFile(element));
        yield convertResourceJson(projectRoot);
        yield updateResourceConfigFileContent(filename);
        merge.output();
    });
}
exports.build = build;
function updateResourceConfigFileContent(filename) {
    return __awaiter(this, void 0, void 0, function* () {
        let content = JSON.stringify(_1.ResourceConfig.config, null, "\t");
        yield fs.writeFileAsync(filename, content, "utf-8");
        return content;
        // var c = ResourceConfig.config;
        // let content = await updateResourceConfigFileContent_2(filename, "exports.resources", c.resources);
        // content = await updateResourceConfigFileContent_2(filename, "exports.groups", c.groups);
        // content = await updateResourceConfigFileContent_2(filename, "exports.alias", c.alias);
        // return content;
    });
}
exports.updateResourceConfigFileContent = updateResourceConfigFileContent;
function updateResourceConfigFileContent_2(filename, matcher, data) {
    return __awaiter(this, void 0, void 0, function* () {
        let content = yield c.publish(filename, matcher, data);
        yield fs.writeFileAsync(filename, content, "utf-8");
        return content;
    });
}
function convertResourceJson(projectRoot) {
    return __awaiter(this, void 0, void 0, function* () {
        let filename = path.join(projectRoot, "resource/default.res.json");
        if (!fs.existsSync(filename)) {
            filename = path.join(projectRoot, "resource/resource.json");
        }
        if (!fs.existsSync(filename)) {
            return;
        }
        let config = _1.ResourceConfig.config;
        let resourceJson = yield fs.readJSONAsync(filename);
        // let resourceJson: original.Info = await fs.readJSONAsync(resourceJsonPath);
        for (let r of resourceJson.resources) {
            config.alias[r.name] = r.url;
            let file = _1.ResourceConfig.getFile(r.url);
            for (var resource_custom_key in r) {
                if (resource_custom_key == "url" || resource_custom_key == "type" || resource_custom_key == "name") {
                    continue;
                }
                else if (resource_custom_key == "subkeys") {
                    var subkeysArr = r.subkeys.split(",");
                    for (let subkey of subkeysArr) {
                        // if (!obj.alias[subkeysArr[i]]) {
                        config.alias[subkey] = r.url + "#" + subkey;
                    }
                }
                else {
                    if (!file) {
                    }
                    else if (typeof file != "string") {
                        file[resource_custom_key] = r[resource_custom_key];
                    }
                    else {
                        console.warn(`missing properties ${resource_custom_key} in ${file}`);
                    }
                }
            }
        }
        for (let group of resourceJson.groups) {
            config.groups[group.name] = group.keys.split(",");
        }
        return _1.ResourceConfig.config;
    });
}
exports.convertResourceJson = convertResourceJson;
var merge;
(function (merge_1) {
    let mergeCollection = {};
    function walk(f) {
        if (_1.ResourceConfig.mergeSelector) {
            let merge = _1.ResourceConfig.mergeSelector(f);
            if (merge) {
                let mergeFile = merge.path;
                merge.path = f;
                let type = _1.ResourceConfig.typeSelector(f);
                if (!type) {
                    throw new Error(`missing merge type : ${merge.path}`);
                }
                if (!mergeCollection[mergeFile]) {
                    mergeCollection[mergeFile] = [];
                }
                mergeCollection[mergeFile].push(merge);
            }
        }
    }
    merge_1.walk = walk;
    function output() {
        for (let mergeFile in mergeCollection) {
            let outputJson = {};
            let sourceFiles = mergeCollection[mergeFile];
            if (_1.ResourceConfig.typeSelector(mergeFile) == "mergeJson") {
                sourceFiles.map(s => {
                    let sourcePath = path.join(resourcePath, s.path);
                    let json = fs.readJSONSync(sourcePath);
                    outputJson[s.alias] = json;
                });
            }
            fs.writeFileSync(path.join(resourcePath, mergeFile), JSON.stringify(outputJson));
        }
    }
    merge_1.output = output;
})(merge || (merge = {}));
